@@ -1,18 +1,20 @@
 /*
- * s390-tools/tunedasd/include/disk.h
- *   Functions to handle dasd specific operations.
+ * tunedasd - Adjust tunable parameters on DASD devices
  *
- * (C) Copyright IBM Corp. 2004
+ * Functions to handle dasd specific operations
  *
- * Author(s): Horst Hummel (horst.hummel@de.ibm.com)
+ * Copyright IBM Corp. 2004, 2017
+ *
+ * s390-tools is free software; you can redistribute it and/or modify
+ * it under the terms of the MIT license. See LICENSE for details.
  */
 
 #ifndef DISK_H
 #define DISK_H
 
+#include <linux/hdreg.h>
 #include <stdint.h>
 #include <sys/types.h>
-#include <linux/hdreg.h>
 
 
 int check_cache (char* cache);
@@ -25,8 +27,10 @@ int disk_set_cache (char* device, char* cache, char* no_cyl);
 int disk_reserve (char* device);
 int disk_release (char* device);
 int disk_slock (char* device);
+int disk_query_reserve_status(char* device);
 int disk_profile (char* device, char* prof_item);
-int disk_reset_prof (char* device);
+int disk_reset_prof(char *device);
+int disk_reset_chpid(char *device, char *chpid);
 
 #endif /* not DISK_H */
 
