@@ -58,11 +58,50 @@ static void add_early_removed(struct util_list *selected)
 	}
 }
 
+static void add_pers_removed(struct util_list *strlist)
+{
+	int i, j;
+	struct devtype *dt;
+	struct subtype *st;
+	struct device *dev;
+
+	for (i = 0; devtypes[i]; i++) {
+		dt = devtypes[i];
+		for (j = 0; dt->subtypes[j]; j++) {
+			st = dt->subtypes[j];
+			util_list_iterate(&st->devices->hash.list, dev) {
+				if (dev->persistent.deconfigured) {
+					strlist_add(strlist, "%s %s",
+						    dev->subtype->devname, dev->id);
+				}
+			}
+		}
+	}
+}
+
+static bool is_zdev_early_0(struct selected_dev_node *sel)
+{
+	struct setting *s;
+	struct device *dev;
+
+	dev = device_list_find(sel->st->devices, sel->id, NULL);
+	if (!dev)
+		return false;
+	s = setting_list_find(dev->persistent.settings,
+			      internal_attr_early.name);
+	if (!s)
+		return false;
+	if (s->specified && strcmp(s->value, "0") == 0)
+		return true;
+
+	return false;
+}
+
 /* Determine if initial RAM-disk needs updating. If so, run the corresponding
  * scripts if available. */
-exit_code_t root_check(void)
+exit_code_t initrd_check(bool all_pers)
 {
-	struct util_list *selected, *params, *mod = NULL;
+	struct util_list *selected, *params, *mod = strlist_new();
 	struct selected_dev_node *sel;
 	struct device *dev;
 	char *params_str;
@@ -76,6 +115,20 @@ exit_code_t root_check(void)
 	/* Get list of devices that provide the root device or require
 	 * early configuration. */
 	selected = selected_dev_list_new();
+
+	if (all_pers) {
+		/* Add all persistently configured devices. */
+		select = select_opts_new();
+		select->configured = 1;
+		select_devices(select, selected, 1, 0, 0, config_persistent,
+			       scope_mandatory, err_ignore);
+		select_opts_free(select);
+
+		/* Ensure that removed devices are considered. */
+		add_pers_removed(mod);
+		goto check_mod;
+	}
+
 	/* First add devices that had zdev:early removed or changed to 0.
 	 * The subsequent call to select_devices() will filter out any
 	 * duplicates. */
@@ -95,8 +148,8 @@ exit_code_t root_check(void)
 		       err_ignore);
 	select_opts_free(select);
 
+check_mod:
 	/* Determine if any of the devices or device types has been modified. */
-	mod = strlist_new();
 	util_list_iterate(selected, sel) {
 		dt = sel->st->devtype;
 
@@ -127,17 +180,22 @@ exit_code_t root_check(void)
 		goto out;
 	}
 
-	/* Ask for confirmation. */
-	if (!confirm("Update initial RAM-disk now?")) {
-		rc = EXIT_ABORTED;
-		goto out;
+	if (!all_pers) {
+		/* Ask for confirmation. */
+		if (!confirm("Update initial RAM-disk now?")) {
+			rc = EXIT_ABORTED;
+			goto out;
+		}
 	}
 
 	/* Build the command line. */
 	params = strlist_new();
 	util_list_iterate(selected, sel) {
-		strlist_add(params, "%s", sel->st->name);
-		strlist_add(params, "%s", sel->id);
+		/* From the selected list, remove the devices with zdev:early=0 */
+		if (!is_zdev_early_0(sel)) {
+			strlist_add(params, "%s", sel->st->name);
+			strlist_add(params, "%s", sel->id);
+		}
 	}
 	params_str = strlist_flatten(params, " ");
 	strlist_free(params);
