@@ -345,6 +345,7 @@ static exit_code_t udev_read_zfcp_lun_rule(const char *filename,
 		in_fc,
 		in_scsi,
 	} state = none;
+	bool empty_rule = true;
 
 	rc = udev_read_file(filename, &file);
 	if (rc)
@@ -374,16 +375,22 @@ static exit_code_t udev_read_zfcp_lun_rule(const char *filename,
 				state = in_scsi;
 				node = zfcp_lun_node_from_entry(entry, node,
 								list);
+				if (node)
+					empty_rule = false;
 			}
 			break;
 		case in_fc:
 			node = zfcp_lun_node_from_entry(entry, node, list);
-			if (node)
+			if (node) {
 				add_fc_setting_from_entry(entry, node);
+				empty_rule = false;
+			}
 			break;
 		case in_scsi:
-			if (node)
+			if (node) {
 				add_scsi_setting_from_entry(entry, node);
+				empty_rule = false;
+			}
 			break;
 		}
 	}
@@ -393,6 +400,9 @@ static exit_code_t udev_read_zfcp_lun_rule(const char *filename,
 out:
 	udev_free_file(file);
 
+	if (empty_rule)
+		warn_once("Warning: Invalid udev rule: %s\n", filename);
+
 	return rc;
 }
 
@@ -475,11 +485,17 @@ static void zfcp_lun_node_to_state(struct zfcp_lun_node *node,
 	struct attrib *a;
 	char *name;
 
-	state->exists = 1;
 	state->modified = 0;
 	state->deconfigured = 0;
 	state->definable = 0;
 
+	if (!node) {
+		state->exists = 0;
+		return;
+	}
+
+	state->exists = 1;
+
 	util_list_iterate(&node->fc_settings->list, s) {
 		a = attrib_find(attribs, s->name);
 		setting_list_add(state->settings,
@@ -519,10 +535,7 @@ exit_code_t udev_zfcp_lun_read_device(struct device *dev, bool autoconf)
 		goto out;
 
 	node = zfcp_lun_node_find(luns, dev->devid);
-	if (node)
-		zfcp_lun_node_to_state(node, st->dev_attribs, state);
-	else
-		rc = EXIT_DEVICE_NOT_FOUND;
+	zfcp_lun_node_to_state(node, st->dev_attribs, state);
 
 out:
 	zfcp_lun_node_list_free(luns);
