@@ -21,6 +21,8 @@
 #include "lib/zt_common.h"
 #include "lib/util_part.h"
 #include "lib/util_path.h"
+#include "boot/s390.h"
+#include "stage3.h"
 
 #include "boot.h"
 #include "bootmap.h"
@@ -507,7 +509,8 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	size_t stage3_params_size;
 	const char *comp_name[10];
 	size_t signature_size;
-	int offset, flags = 0;
+	int offset;
+	uint64_t flags = 0;
 	void *stage3_params;
 	struct stat stats;
 	void *signature;
@@ -946,6 +949,7 @@ build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
 {
 	disk_blockptr_t* table;
 	int entries, component_header;
+	int is_secure;
 	int i;
 	int rc;
 
@@ -1017,13 +1021,18 @@ build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
 						component_header_ipl;
 					printf("\n");
 				}
+				if (job->is_secure != SECURE_BOOT_UNDEFINED)
+					is_secure = job->is_secure;
+				else
+					is_secure =
+					      job->data.menu.entry[i].is_secure;
 				rc = add_ipl_program(fd,
 					&job->data.menu.entry[i].data.ipl,
 					&table[job->data.menu.entry[i].pos],
 					verbose || job->command_line,
 					job->add_files,	component_header,
 						     info, &job->target,
-						     job->is_secure);
+						     is_secure);
 				break;
 			case job_print_usage:
 			case job_print_version:
@@ -1134,6 +1143,12 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 			     disk_get_type_name(info->type));
 		goto out_disk_free_info;
 	}
+	/* Check if secure boot was enabled only for SCSI */
+	if (job->is_secure == SECURE_BOOT_ENABLED &&
+	    info->type != disk_type_scsi) {
+		error_reason("Secure boot forced for non-SCSI disk type");
+		goto out_disk_free_info;
+	}
 	if (verbose) {
 		printf("Target device information\n");
 		disk_print_info(info);
@@ -1274,9 +1289,7 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		break;
 	}
 	if (dry_run) {
-		if (remove(filename) == -1)
-			fprintf(stderr, "Warning: could not remove temporary "
-				"file %s!\n", filename);
+		misc_free_temp_file(filename);
 	} else if (job->id != job_dump_partition) {
 		/* Rename to final bootmap name */
 		mapname = misc_make_path(job->target.bootmap_dir,
@@ -1303,6 +1316,8 @@ out_disk_free_info:
 	disk_free_info(info);
 out_close_fd:
 	close(fd);
+	if (job->id != job_dump_partition)
+		misc_free_temp_file(filename);
 out_free_filename:
 	free(filename);
 	return -1;
